@@ -1,7 +1,7 @@
 
-/*  $Header: /cvs/gnome/gcalctool/gcalctool/display.c,v 1.57 2005/02/07 16:38:58 richb Exp $
+/*  $Header: /cvs/gnome/gcalctool/gcalctool/display.c,v 1.64 2006/08/07 23:53:21 richb Exp $
  *
- *  Copyright (c) 1987-2005 Sun Microsystems, Inc. All Rights Reserved.
+ *  Copyright (c) 1987-2006 Sun Microsystems, Inc. All Rights Reserved.
  *           
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -26,9 +26,9 @@
 #include <gdk/gdkkeysyms.h>
 #include "extern.h"
 #include "mpmath.h"
+#include "functions.h"
 
 static char *make_eng_sci(int *, int);
-static char *make_fixed(int *, int, int);
 
 /* Add in the thousand separators characters if required and if we are
  * currently in the decimal numeric base, use the "right" radix character.
@@ -116,7 +116,7 @@ clear_display(int initialise)
     v->toclear = 1;
     i = 0;
     mpcim(&i, v->MPdisp_val);
-    STRCPY(v->display, make_number(v->MPdisp_val, v->base, FALSE, FALSE));
+    STRCPY(v->display, make_number(v->MPdisp_val, v->base, FALSE));
     set_display(v->display, FALSE);
 
     if (initialise == TRUE) {
@@ -152,7 +152,8 @@ initialise()
     /* TODO: perhaps this function should be renamed to reset. */
 
     int i;
-  
+    struct exprm_state *e;
+ 
     v->error         = 0;           /* Currently no display error. */
     v->cur_op        = '?';         /* No arithmetic operator defined yet. */
     v->old_cal_value = '?';
@@ -164,8 +165,9 @@ initialise()
   
     v->new_input = 1;               /* Value zero is on calculator display */
 
-    free(v->expression);
-    v->expression = NULL;
+    e = get_state();
+    free(e->expression);
+    e->expression = NULL;
 }
 
 
@@ -173,8 +175,8 @@ initialise()
  * maximum number of digits specified.
  */
 
-static char *
-make_fixed(int *MPnumber, int base, int cmax)
+char *
+make_fixed(int *MPnumber, char *str, int base, int cmax, int toclear)
 {
     char half[4], *optr;
     int MP1base[MP_SIZE], MP1[MP_SIZE], MP2[MP_SIZE], MPval[MP_SIZE];
@@ -182,7 +184,7 @@ make_fixed(int *MPnumber, int base, int cmax)
     int ddig;                   /* Number of digits to left of decimal sep. */
     int dval, n;
  
-    optr = v->fnum;
+    optr = str;
     mpabs(MPnumber, MPval);
     n = 0;
     mpcim(&n, MP1);
@@ -228,28 +230,30 @@ make_fixed(int *MPnumber, int base, int cmax)
         mpaddi(MPval, &dval, MPval);
     }    
     *optr++ = '\0';
-    v->toclear = 1;
+    if (toclear == TRUE) {
+        v->toclear = 1;
+    }
     v->pointed = 0;
 
     if (!v->show_zeroes && v->accuracy != 0) {
-        optr = v->fnum + strlen(v->fnum) - 1;
+        optr = str + strlen(str) - 1;
         while (*optr == '0') {
             optr--;
         }
-    if (optr < v->fnum || *optr != '.') {
+    if (optr < str || *optr != '.') {
         optr++;
     }
         *optr = '\0';
     }
 
-    return(v->fnum);
+    return(str);
 }
 
 
 /* Convert MP number to character string in the given base. */
 
 char *
-make_number(int *MPnumber, int base, BOOLEAN mkFix, BOOLEAN ignoreError)
+make_number(int *MPnumber, int base, BOOLEAN ignoreError)
 {
     double number, val;
 
@@ -271,7 +275,7 @@ make_number(int *MPnumber, int base, BOOLEAN mkFix, BOOLEAN ignoreError)
         (v->dtype == FIX && val != 0.0 && (val > max_fix[base]))) {
         return(make_eng_sci(MPnumber, base));
     } else {
-        return(make_fixed(MPnumber, base, MAX_DIGITS));
+        return(make_fixed(MPnumber, v->fnum, base, MAX_DIGITS, TRUE));
     }
 }
 
@@ -340,7 +344,7 @@ make_eng_sci(int *MPnumber, int base)
         }
     }
  
-    STRCPY(fixed, make_fixed(MPmant, base, MAX_DIGITS-6));
+    STRCPY(fixed, make_fixed(MPmant, v->fnum, base, MAX_DIGITS-6, TRUE));
     len = strlen(fixed);
     for (i = 0; i < len; i++) {
         *optr++ = fixed[i];
@@ -394,6 +398,8 @@ MPstr_to_num(char *str, enum base_type base, int *MPval)
     int i, inum;
     int exp      = 0;
     int exp_sign = 1;
+    char *lnp = get_localized_numeric_point();
+    assert(lnp);
 
     i = 0;
     mpcim(&i, MPval);
@@ -406,7 +412,7 @@ MPstr_to_num(char *str, enum base_type base, int *MPval)
         optr++;
     }
 
-    if (*optr == '.') {
+    if (*optr == '.' || *optr == *lnp) {
         optr++;
         for (i = 1; (inum = char_val(*optr)) >= 0; i++) {
             mppwr(MPbase, &i, MP1);
@@ -442,7 +448,7 @@ MPstr_to_num(char *str, enum base_type base, int *MPval)
 /* Append the latest parenthesis char to the display item. */
 
 void
-paren_disp(char c)
+paren_disp(int c)
 {
     int i, n;
 
@@ -566,7 +572,7 @@ void
 show_display(int *MPval)
 {
     if (!v->error) {
-        STRCPY(v->display, make_number(MPval, v->base, TRUE, FALSE));
+        STRCPY(v->display, make_number(MPval, v->base, FALSE));
         set_display(v->display, FALSE);
     }
 }
@@ -585,24 +591,25 @@ refresh_display()
     show_display(v->MPdisp_val);
     break;
     
-  case exprs:
-    if (v->expression &&
-	strlen(v->expression)) {
-      char *e = gc_strdup(v->expression);
-      char *ans = make_number(v->e.ans, v->base, TRUE, FALSE);
+  case exprs: {
+	struct exprm_state *e = get_state();
+    if (e->expression &&
+		strlen(e->expression)) {
+      char *str = gc_strdup(e->expression);
+      char *ans = make_number(e->ans, v->base, FALSE);
       localize_number(localized, ans);
       
-      str_replace(&e, "Ans", localized);
+      str_replace(&str, "Ans", localized);
       //set_display(e, FALSE);
-      write_display(e);
-      free(e);
+      write_display(str);
+      free(str);
     } else {
       /* set_display("", FALSE); */
       int MP1[MP_SIZE];
       do_zero(MP1);
       show_display(MP1);
     }
-    
+  }    
     break;
     
   default:
